@@ -23,7 +23,7 @@ def bs(byte_array):
     return ''.join([chr(c) for c in byte_array])
 
 DEBUG = False
-__version__ = '0.4.3'
+__version__ = '0.4.4'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -239,8 +239,17 @@ class XSQLVAR:
         return (h, m, s, (n % 10000) * 100)
 
     def value(self, raw_value):
-        if self.sqltype in (SQL_TYPE_TEXT, SQL_TYPE_VARYING):
-            return self.bytes_to_str(raw_value)
+        if self.sqltype == SQL_TYPE_TEXT:
+            if self.sqlsubtype in (4, 69):  # UTF8 and GB18030 
+                reallength = self.sqllen // 4 
+                return self.bytes_to_str(raw_value)[:reallength]
+            elif self.sqlsubtype == 3:      # UNICODE_FSS 
+                reallength = self.sqllen // 3
+                return self.bytes_to_str(raw_value)[:reallength]
+            else:
+                return self.bytes_to_str(raw_value) 
+        elif self.sqltype == SQL_TYPE_VARYING:
+                return self.bytes_to_str(raw_value) 
         elif self.sqltype in (SQL_TYPE_SHORT, SQL_TYPE_LONG, SQL_TYPE_INT64):
             n = bytes_to_bint(raw_value)
             if self.sqlscale:
@@ -393,8 +402,7 @@ class cursor:
 
         cooked_params = []
         for param in params:        # Convert str/bytes parameter to blob id
-            if ((PYTHON_MAJOR_VER==3 and type(param) == str) or 
-                            (PYTHON_MAJOR_VER==2 and type(param) == unicode)):
+            if type(param) == str: 
                 param = self.connection.str_to_bytes(param)
             elif ((PYTHON_MAJOR_VER==3 and type(param) != bytes) or 
                             (PYTHON_MAJOR_VER==2 and type(param) != str)):
@@ -565,6 +573,8 @@ class BaseConnect:
     op_detach = 21
     op_transaction = 29
     op_commit = 30
+    op_commit_retaining = 50
+    op_rollback_retaining = 86
     op_rollback = 31
     op_open_blob = 35
     op_get_segment = 36
@@ -585,7 +595,51 @@ class BaseConnect:
     op_service_start = 85
 
     charset_map = {
-        'UNICODE_FSS' : 'utf-8',
+    # DB CHAR SET NAME    :   PYTHON CODEC NAME (CANONICAL)
+    # ---------------------------------------------------------------------------
+    'OCTETS'              :   None, # Allow to pass through unchanged.
+    'UNICODE_FSS'         :   'utf_8',
+    'UTF8'                :   'utf_8', # (Firebird 2.0+)
+    'SJIS_0208'           :   'shift_jis',
+    'EUCJ_0208'           :   'euc_jp',
+    'DOS737'              :   'cp737',
+    'DOS437'              :   'cp437',
+    'DOS850'              :   'cp850',
+    'DOS865'              :   'cp865',
+    'DOS860'              :   'cp860',
+    'DOS863'              :   'cp863',
+    'DOS775'              :   'cp775',
+    'DOS862'              :   'cp862',
+    'DOS864'              :   'cp864',
+    'ISO8859_1'           :   'iso8859_1',
+    'ISO8859_2'           :   'iso8859_2',
+    'ISO8859_3'           :   'iso8859_3',
+    'ISO8859_4'           :   'iso8859_4',
+    'ISO8859_5'           :   'iso8859_5',
+    'ISO8859_6'           :   'iso8859_6',
+    'ISO8859_7'           :   'iso8859_7',
+    'ISO8859_8'           :   'iso8859_8',
+    'ISO8859_9'           :   'iso8859_9',
+    'ISO8859_13'          :   'iso8859_13',
+    'KSC_5601'            :   'euc_kr',
+    'DOS852'              :   'cp852',
+    'DOS857'              :   'cp857',
+    'DOS861'              :   'cp861',
+    'DOS866'              :   'cp866',
+    'DOS869'              :   'cp869',
+    'WIN1250'             :   'cp1250',
+    'WIN1251'             :   'cp1251',
+    'WIN1252'             :   'cp1252',
+    'WIN1253'             :   'cp1253',
+    'WIN1254'             :   'cp1254',
+    'BIG_5'               :   'big5',
+    'GB_2312'             :   'gb2312',
+    'WIN1255'             :   'cp1255',
+    'WIN1256'             :   'cp1256',
+    'WIN1257'             :   'cp1257',
+    'KOI8-R'              :   'koi8_r', # (Firebird 2.0+)
+    'KOI8-U'              :   'koi8_u', # (Firebird 2.0+)
+    'WIN1258'             :   'cp1258', # (Firebird 2.0+)
     }
 
     def str_to_bytes(self, s):
@@ -611,8 +665,7 @@ class BaseConnect:
         values = bs([])
         for p in params:
             t = type(p)
-
-            if t == str or t == unicode:
+            if t == str:
                 v = self.str_to_bytes(p)
                 nbytes = len(v)
                 pad_length = ((4-nbytes) & 3)
@@ -791,6 +844,13 @@ class BaseConnect:
         p.pack_int(self.trans_handle)
         send_channel(self.sock, p.get_buffer())
 
+    @wire_operation
+    def _op_commit_retaining(self):
+        p = xdrlib.Packer()
+        p.pack_int(self.op_commit_retaining)
+        p.pack_int(self.trans_handle)
+        send_channel(self.sock, p.get_buffer())
+
     @wire_operation
     def _op_rollback(self):
         p = xdrlib.Packer()
@@ -798,6 +858,12 @@ class BaseConnect:
         p.pack_int(self.trans_handle)
         send_channel(self.sock, p.get_buffer())
 
+    @wire_operation
+    def _op_rollback_retaining(self):
+        p = xdrlib.Packer()
+        p.pack_int(self.op_rollback_retaining)
+        p.pack_int(self.trans_handle)
+        send_channel(self.sock, p.get_buffer())
 
     @wire_operation
     def _op_allocate_statement(self):
@@ -1002,15 +1068,23 @@ class BaseConnect:
         (h, oid, buf) = self._op_response()
         self.trans_handle = h
     
-    def commit(self):
-        self._op_commit()
-        (h, oid, buf) = self._op_response()
-        delattr(self, "trans_handle")
+    def commit(self, retaining=False):
+        if retaining:
+            self._op_commit_retaining()
+            (h, oid, buf) = self._op_response()
+        else:
+            self._op_commit()
+            (h, oid, buf) = self._op_response()
+            delattr(self, "trans_handle")
 
-    def rollback(self):
-        self._op_rollback()
-        (h, oid, buf) = self._op_response()
-        delattr(self, "trans_handle")
+    def rollback(self, retaining=False):
+        if retaining:
+            self._op_rollback_retaining()
+            (h, oid, buf) = self._op_response()
+        else:
+            self._op_rollback()
+            (h, oid, buf) = self._op_response()
+            delattr(self, "trans_handle")
 
     def close(self):
         if not hasattr(self, "db_handle"):
